unit FillEdit;

//  (C) Sergey Dolinin, Perm, 2000
//  mailto: dolinin@energos.perm.ru

//  property AllowAutoFill: boolean - / 
//  property Lines: TStrings        - c,    

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TFillEdit = class(TEdit)
  private
    FItems: TStrings;
    FAllowAutoFill: boolean;
    DisableFill: boolean;
    FSafeRegister: boolean;
    procedure SetItems(const Value: TStrings);
    { Private declarations }
  protected
    { Protected declarations }
    procedure Change; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure ContextClear(var Message: TMessage); message WM_CLEAR;
    procedure ContextCut(var Message: TMessage); message WM_CUT;
    procedure ContextPaste(var Message: TMessage); message WM_PASTE;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Lines: TStrings read FItems write SetItems;
    property AllowAutoFill: boolean read FAllowAutoFill write FAllowAutoFill default True;
    property RegisterFromLines: boolean read FSafeRegister write FSafeRegister default True;
  end;

  TFillComboBox = class(TComboBox)
  private
    FAllowAutoFill: boolean;
    DisableFill: boolean;
    FSafeRegister: boolean;
    { Private declarations }
  protected
    { Protected declarations }
    procedure Change; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure ContextClear(var Message: TMessage); message WM_CLEAR;
    procedure ContextCut(var Message: TMessage); message WM_CUT;
    procedure ContextPaste(var Message: TMessage); message WM_PASTE;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property AllowAutoFill: boolean read FAllowAutoFill write FAllowAutoFill default True;
    property RegisterFromLines: boolean read FSafeRegister write FSafeRegister default True;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Custom', [TFillEdit, TFillComboBox]);
end;

{ TFillEdit }

procedure TFillEdit.Change;
var i, l: integer;
begin
  inherited Change;
  if AllowAutoFill and (not DisableFill) then
  for i := 0 to Lines.Count - 1 do
    if Pos(AnsiUpperCase(Text), AnsiUpperCase(Lines[i])) = 1 then
    begin
      l := Length(Text);
      if RegisterFromLines
      then Text := Lines[i]
      else Text := Text + Copy(Lines[i], l + 1, Length(Lines[i]) - l + 1);
      SelStart := l;
      SelLength := Length(Text) - l;
      Break;
    end;
end;

procedure TFillEdit.ContextClear(var Message: TMessage);
begin
  DisableFill := True;
  inherited;
end;

procedure TFillEdit.ContextCut(var Message: TMessage);
begin
  DisableFill := True;
  inherited;
end;

procedure TFillEdit.ContextPaste(var Message: TMessage);
begin
  DisableFill := True;
  inherited;
end;

constructor TFillEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TStringList.Create;
  FAllowAutoFill := True;
  FSafeRegister := True;
end;

destructor TFillEdit.Destroy;
begin
  FItems.Free;
  inherited destroy;
end;

procedure TFillEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  DisableFill := (Key = VK_Delete) or (Key = 8);
  inherited KeyDown(Key, Shift);
end;

procedure TFillEdit.SetItems(const Value: TStrings);
begin
  FItems.Assign(Value);
end;

{ TFillComboBox }

procedure TFillComboBox.Change;
var i, l: integer;
begin
  inherited Change;
  if (Style in [csSimple, csDropDown]) and AllowAutoFill and (not DisableFill) then
  for i := 0 to Items.Count - 1 do
    if Pos(AnsiUpperCase(Text), AnsiUpperCase(Items[i])) = 1 then
    begin
      l := Length(Text);
      if RegisterFromLines
      then Text := Items[i]
      else Text := Text + Copy(Items[i], l + 1, Length(Items[i]) - l + 1);
      SelStart := l;
      SelLength := Length(Text) - l;
      Break;
    end;
end;

procedure TFillComboBox.ContextClear(var Message: TMessage);
begin
  DisableFill := True;
  inherited;
end;

procedure TFillComboBox.ContextCut(var Message: TMessage);
begin
  DisableFill := True;
  inherited;
end;

procedure TFillComboBox.ContextPaste(var Message: TMessage);
begin
  DisableFill := True;
  inherited;
end;

constructor TFillComboBox.Create(AOwner: TComponent);
begin
  inherited;
  FAllowAutoFill := True;
  FSafeRegister := True;
end;

procedure TFillComboBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  DisableFill := (Key = VK_Delete) or (Key = 8);
  inherited KeyDown(Key, Shift);
end;

end.


